from .common import Benchmark, Estimator
from .datasets import _olivetti_faces_dataset
from sklearn.metrics.pairwise import euclidean_distances
from sklearn.metrics import homogeneity_score, silhouette_score, adjusted_mutual_info_score 
from sklearn.cluster import AffinityPropagation
from skapex import LeveragedAffinityPropagation

class FacesBaselineAP(Estimator, Benchmark):
    """
    Benchmarks for Affinity Propagation, used as baseline.
    """
    def setup_cache(self):
        super().setup_cache()

    def make_data(self, params):
        return _olivetti_faces_dataset()
    
    param_names = ["damping"]
    params = ([0.5, 0.8],)

    def make_estimator(self, params):
        damping, *_ = params
        return AffinityPropagation(damping=damping)    
    
    def make_scorers(self):
        pass

    def track_homogeneity(self, *args):
        return float(homogeneity_score(self.y, self.estimator.labels_))
    
    def track_silhouette(self, *args):
        return float(silhouette_score(self.X, self.estimator.labels_))
    
    def track_adjusted_variation_info(self, *args):
        return float(adjusted_mutual_info_score(self.y, self.estimator.labels_))


class FacesLAP(Estimator, Benchmark):
    """
    Benchmarks for Leveraged Affinity Propagation.
    """
    def setup_cache(self):
        super().setup_cache()
        
    def make_data(self, params):
        return _olivetti_faces_dataset()
    
    param_names = ["fraction", "sweeps", "seed"]
    params = ([0.1, 0.2, 0.3], [1, 2, 3, 4], [0, 1, 2, 3, 4])
    
    def make_estimator(self, params):
        fraction, sweeps, seed = params
        estimator = LeveragedAffinityPropagation(
            simf=lambda x,y: -euclidean_distances(x, y, squared=True),
            fraction=fraction,
            sweeps=sweeps,
            random_state=seed,
        )
        return estimator
    
    def make_scorers(self):
        pass

    def track_homogeneity(self, *args):
        return float(homogeneity_score(self.y, self.estimator.labels_))
    
    def track_silhouette(self, *args):
        return float(silhouette_score(self.X, self.estimator.labels_))
    
    def track_adjusted_variation_info(self, *args):
        return float(adjusted_mutual_info_score(self.y, self.estimator.labels_))
