from .common import Benchmark, Estimator
from .datasets import _mnist_fashion_dataset
from sklearn.metrics.pairwise import euclidean_distances
from sklearn.metrics import homogeneity_score, silhouette_score, adjusted_mutual_info_score 
from sklearn.cluster import AffinityPropagation
from skapex import LeveragedAffinityPropagation
from abc import ABC

class FashionBenchmark(Estimator, Benchmark, ABC):
    n_samples = [10, 100, 1000, 5000]

    def setup_cache(self):
        super().setup_cache()

    def make_data(self, params):
        *_, n_samples = params
        X, X_val, y, y_val = _mnist_fashion_dataset()
        return X[:n_samples], _, y[:n_samples], _
    
    def make_scorers(self):
        pass

    def track_homogeneity(self, *args):
        return float(homogeneity_score(self.y, self.estimator.labels_))
    
    def track_silhouette(self, *args):
        return float(silhouette_score(self.X, self.estimator.labels_))
    
    def track_adjusted_variation_info(self, *args):
        return float(adjusted_mutual_info_score(self.y, self.estimator.labels_))


class FashionBaselineAP(FashionBenchmark):
    """
    Benchmarks for Affinity Propagation, used as baseline.
    """
    def setup_cache(self):
        super().setup_cache()
    param_names = ["n_samples"]
    @property
    def params(self): return (super().n_samples,)

    def make_estimator(self, params):
        return AffinityPropagation()    


class FashionLAP(FashionBenchmark):
    """
    Benchmarks for Leveraged Affinity Propagation.
    """
    def setup_cache(self):
        super().setup_cache()
    param_names = ["fraction", "sweeps", "seed", "n_samples"]
    @property
    def params(self): return ([0.005, 0.02, 0.1], [1, 2, 3], [0, 1, 2], super().n_samples)
    
    def make_estimator(self, params):
        fraction, sweeps, seed, _ = params
        estimator = LeveragedAffinityPropagation(
            simf=lambda x,y: -euclidean_distances(x, y, squared=True),
            fraction=fraction,
            sweeps=sweeps,
            random_state=seed,
        )
        return estimator
