from .common import Benchmark, Estimator
from .datasets import _blobs_dataset
from sklearn.metrics.pairwise import euclidean_distances
from sklearn.metrics import homogeneity_score, silhouette_score, adjusted_mutual_info_score 
from sklearn.cluster import AffinityPropagation
from skapex import LeveragedAffinityPropagation
from abc import ABC

class SyntheticBenchmark(Estimator, Benchmark, ABC):
    n_samples = [10, 1000, 2000, 5000]

    def setup_cache(self):
        super().setup_cache()

    def make_data(self, params):
        *_, n_samples = params
        return _blobs_dataset(n_samples=n_samples, n_features=10, n_clusters=5)
    
    def make_scorers(self):
        pass

    def track_homogeneity(self, *args):
        return float(homogeneity_score(self.y, self.estimator.labels_))
    
    def track_silhouette(self, *args):
        return float(silhouette_score(self.X, self.estimator.labels_))
    
    def track_adjusted_variation_info(self, *args):
        return float(adjusted_mutual_info_score(self.y, self.estimator.labels_))

class SyntheticBaselineAP(SyntheticBenchmark):
    """
    Benchmarks for Affinity Propagation, used as baseline.
    """
    def setup_cache(self):
        super().setup_cache()
    param_names = ["n_samples"]
    @property
    def params(self): return (super().n_samples,)

    def make_estimator(self, params):
        return AffinityPropagation()
    


class SyntheticLAP(SyntheticBenchmark):
    """
    Benchmarks for Leveraged Affinity Propagation.
    """
    def setup_cache(self):
        super().setup_cache()
    param_names = ["fraction", "sweeps", "seed", "n_samples"]
    @property
    def params(self): return ([0.005, 0.02, 0.1], [1, 2, 5], [0, 1, 3], super().n_samples)
    
    def make_estimator(self, params):
        fraction, sweeps, seed, _ = params
        estimator = LeveragedAffinityPropagation(
            simf=lambda x,y: -euclidean_distances(x, y, squared=True),
            fraction=fraction,
            sweeps=sweeps,
            random_state=seed,
        )
        return estimator
